package wm_main;

###################################################################
#
# Licensed Materials - Property of IBM
# 5724-T05,5724-T06
#
# (C) Copyright IBM Corp. 2003,2008. All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
###################################################################
# Script   : wm_main                                              #
# Function : Driver for all wminstall functions                   #
###################################################################
# (#) ./install/lib/wm_main.pm ; BL_prospect_installer_8.0.6.0.13_20081104  181901

###################################################################
#              Standard/Supplied Perl Modules                     #
###################################################################
use strict;
use Cwd;
use File::Basename;
use File::Copy;
use File::Path;
use Getopt::Long;    # Posix GetOptions

###################################################################
#                      Watchmark Perl Modules                     #
###################################################################
use wm_audit;
use wm_err;
use wm_runtype;
use wm_xml_handler;
use wm_utilities;

###################################################################
#                     Global Vars                                 #
###################################################################

my $spec_exists    = "0";    # 0 means it doesn't
my $spec           = "spec.xml";
my $spec_file      = "";
my $vendor_spec_file = "";
my $log_dir          = "";
my $preview          = "";
my $verbosity        = "";
my $silent           = "";
my $start_dir        = "";
my $spec_dir         = "";
my %responses        = ();

###################################################################
#                 Command line values                             #
###################################################################

my $cl_db_connect    = "";
my $cl_basedir       = "";
my $cl_install_dir   = "";
my $cl_port_base     = "";
my $cl_tmpdir        = "";
my $cl_classpath     = "";
my $cl_spec          = "";
my $verbose          = "";
my $response_file    = "";

my $global_pre_rc    = "0";
my $global_classpath = "";

my %return_code_text = ("0" => "PASS",
                        "1" => "WARNING",
                        "2" => "FAIL",
                       );

###################################################################

sub loop_thru_array ( $@ ) {
###################################################################
#                                                                 #
###################################################################

    my ( $validating, @routine_array ) = @_;

    my $ctr = 0;
    my ($appended_path) = '';

    for ( my $x = 0 ; $x <= $#routine_array ; $x++ ) {
        my ( $spec_type, $script_name, $script_lang ) =
          split ( ":", $routine_array[$x] );
        if ($script_lang ne "java") {
          $appended_path = join "/", $start_dir, $script_name;
          ( $appended_path, my @dummy ) = split ( " ", $appended_path );
          my $print_literal = join " ", $validating, $appended_path;
          if ( $script_name gt "" ) {
            ++$ctr;
            if ( -f $appended_path ) {
              &wm_audit::print_detail( "$print_literal ", "Exists", "Y", "Y",
                                       $verbosity );
            }
            else {
              &wm_audit::print_detail( "$print_literal ", "Missing", "Y", "Y",
                                       $verbosity );
              &wm_err::accum_error( "Validating - $print_literal", "Error",
                                    "Missing" );
            }
          }
        }
    }
    if ( $ctr == 0 ) {
        &wm_audit::print_detail( "$validating ", "<None>", "Y", "Y",
            $verbosity );
    }
}

sub composite_loop ( $@ ) {
###################################################################
#                                                                 #
###################################################################

    my ( $print_literal, @array ) = @_;

    if ( $#array ge 0 ) {
        for ( my $x = 0 ; $x <= $#array ; $x++ ) {
            my ( $spec_type, $script_name, $script_lang ) =
              split ( ":", $array[$x] );
            &wm_audit::print_detail( $print_literal, $script_name, "Y", "Y",
                $verbosity );
        }
    }
    else {
        &wm_audit::print_detail( $print_literal, "<None>", "Y", "Y",
            $verbosity );
    }
}

sub composite_string ( $$ ) {
###################################################################
#                                                                 #
###################################################################

    my ( $print_literal, @passed_string ) = @_;

#    print STDERR ">>>> Called with : Print Literal ($print_literal) Passed String (@passed_string)\n";

    for ( my $x = 0; $x <= $#passed_string; $x++) { 
          my ( $script_name, $script_lang ) = split ( ":", @passed_string[$x] );

         if ( $script_name eq "" ) {
             &wm_audit::print_detail( $print_literal, "<None>", "Y", "Y", "Y");
         }
         else {
             &wm_audit::print_detail( $print_literal, $script_name, "Y", "Y", "Y" );
         }
    }
}

sub print_composite_routines {
###################################################################
#                                                                 #
###################################################################
    my (@tarball_array) = &wm_xml_handler::return_tarballs;

    &wm_audit::print_section( "Processing Steps ", "Y", "Y", "Y" );

    &composite_loop( "Global PRE_VALIDATE  ",
        &wm_xml_handler::return_global_pre_validate );
    &composite_loop( "Global PRE_INSTALL   ",
        &wm_xml_handler::return_global_pre_install );

    for ( my $x = 0 ; $x <= $#tarball_array ; $x++ ) {
        my ( $name, $install_dir, $filename, $process ) =
          split ( ":", $tarball_array[$x] );
        my ($build_dir)     = join "/", $log_dir, $install_dir;
        my ($appended_path) = join "/", $log_dir, $install_dir;

        my ( $print_process ) = "";
        if ( $process eq 'U' ) {
             $print_process = "UPGRADE";
	   }
        else
	  {
            if ( $process eq 'I' ) {
                 $print_process = "INSTALL";
	       }
            else
	      {
                if ( $process eq 'S' ) {
                     $print_process = "SKIP";
		   }
	      }
	  }

        my ( $print_literal ) = "TARBALL - " . $filename;

        &wm_audit::print_detail( $print_literal, $print_process, "Y","Y","Y");
        next if ( $process eq 'S');
        &composite_string( "PRE_VALIDATE  - $install_dir",
            &wm_xml_handler::return_modules_module_prevalidate($x) );
        &composite_string( "PRE_INSTALL   - $install_dir",
            &wm_xml_handler::return_modules_module_preinstall($x) );
        &composite_string( "INSTALL       - $install_dir",
            &wm_xml_handler::return_modules_module_install($x) );
        &composite_string( "POST_INSTALL  - $install_dir",
            &wm_xml_handler::return_modules_module_postinstall($x) );
        &composite_string( "POST_VALIDATE - $install_dir",
            &wm_xml_handler::return_modules_module_postvalidate($x) );

    }
    &composite_loop( "Global POST_INSTALL  ",
        &wm_xml_handler::return_global_post_install );
    &composite_loop( "Global POST_VALIDATE ",
        &wm_xml_handler::return_global_post_validate );

}

sub validate_routines ( $$@ ) {
###################################################################
#                                                                 #
###################################################################
  my ( $literal, $tarfile, @routine ) = @_;

  for ( my $x=0; $x <=$#routine; $x++ ) {
    my ( $script_name, $script_lang ) = split ( ":", @routine[$x] );

    if ( $script_name eq "" ) {
      &wm_audit::print_detail( "$literal ", "<None>", "Y", "Y", $verbosity );
      return (0);
    }

    if ($script_lang ne "java" and $script_lang ne "jython") {
      my ($test_path) = join "/", $start_dir, $tarfile;

      my ( $gtar ) = &wm_utilities::find_gtar($verbosity);
      return(2) if ($gtar == 2);

      my ($return_code) = system("$gtar -tzf $test_path $script_name > /dev/null 2>&1");

      if ( $return_code eq '0') {
        &wm_audit::print_detail( "$literal - $script_name ", "Exists", "Y", "Y",
                                 $verbosity );
      }
      else {
        &wm_audit::print_detail( "$literal - $script_name ", "Missing", "Y",
                                 "Y", $verbosity );
        &wm_err::accum_error( "$literal - $script_name ", "Error", "Missing" );
      }
    }
  }
}

sub process_scripts ( $$$$@ ) {
###################################################################
#                                                                 #
###################################################################
    my (
        $process_dir, $process_string, $global_switch,
        $process,     @process_array
      )
      = @_;

    my ( $DBCONNECT, $BASEDIR, $CORE_INSTALLDIR, $PORTBASE,
        $TMPDIR, $CLASSPATH )
      = &wm_xml_handler::return_core_defaults;

    my ( $spec_type, $script_name, $script_lang ) = "";

    my $return_code = chdir($process_dir);

    if ( $global_pre_rc gt 1 ) {
        if ( $global_switch ne 'Y' ) {
            &wm_audit::print_section( "Abort Processing ", "Y", "Y", "Y" );
            &wm_err::dump_errors($verbosity);
            &wm_audit::close_reports;
            &archive_logs;
            exit(1);
        }
    }

    my $exec_path = "";
    if ( $#process_array ge 0 ) {
        &wm_audit::print_section( "$process_string ", "Y", "Y", "Y" );
        for ( my $x = 0 ; $x <= $#process_array ; $x++ ) {
            my $curr_path = $process_dir;
            if ( $global_switch eq 'Y' ) {
                ( $spec_type, $script_name, $script_lang ) =
                  split ( ":", $process_array[$x] );
                $exec_path = join "/", $start_dir, $script_name;
            }
            else {
                ( $script_name, $script_lang ) =
                  split ( ":", $process_array[$x] );
#               $exec_path = join "/", $curr_path, $script_name;
                $exec_path = join "/",  $script_name;
            }
            next if ( $script_name eq "" );

            my ( $chmod_path, @dummy ) = split ( " ", $exec_path );
            chmod 0755, $chmod_path;

            my $classpath = $ENV{'CLASSPATH'};

            my (@class_array) = split ( ":", $classpath );

            my $temp_string = "";
            my $ob = dirname($ENV{'ORACLE_HOME'});
            my $ov_re = qr{^$ob/([^/]+)};
            my $found_ok_ora_cp = 0;

            #Make sure relative .jar paths are explicit and
            #Check any Oracle class path entries against
            #ORACLE_HOME. ORACLE_HOME and ORACLE_VERS have already
            #been matched in BEGIN of wminstall


            for ( my $y = 0 ; $y <= $#class_array ; $y++ ) {
              if ( $class_array[$y] =~ /\.jar/ ) {
                if ( $class_array[$y] !~ /^\// ) {
                  $class_array[$y] = join "/", $start_dir,
                    $class_array[$y];
                }
              }
              if (!$found_ok_ora_cp and $class_array[$y] =~ /$ov_re/) {
                my $ov = $1;
                $ov =~ s/[^0-9]//g;
                if ($ov ne $ENV{'ORACLE_VERS'}) {
                  &wm_audit::print_section( "FATAL: Oracle installation referenced in CLASSPATH does not match ORACLE_VERS: " . $ENV{'ORACLE_VERS'}, "Y", "Y", "Y" );
                  &wm_err::dump_errors($verbosity);
                  &wm_audit::close_reports;
                  &archive_logs;
                  exit(1);
                }
                else {
                  $found_ok_ora_cp = 1;
                }
              }
              $temp_string .= ":" . $class_array[$y];
            }

            $classpath = $temp_string;

            my ($process_mode) = "";

            if ( $process_string =~ /^INSTALL/ ) {
                if ( $process eq 'I' ) {
                    $process_mode = "-m INSTALL";
                }
                else {
                    if ( $process eq 'U' ) {
                        $process_mode = "-m UPGRADE";
                    }
                }
            }

            my ($temp_file_name) = &wm_utilities::get_temp_file_name("/var/tmp/");
            if ( $script_lang eq "jython" or $script_lang eq "java" ) {
                my ( $script_name, @script_parms ) = split(" ",$exec_path);
                my ( $name, $path, $suffix ) = fileparse($script_name);
                $name =~ s/\.class//g;
                chdir($start_dir); #Make sure we're in our beginning install directory
                my ($parms) = "";
                for ( my $parm_idx = 0; $parm_idx <= $#script_parms; $parm_idx++) {
                     $parms = $parms . " " . $script_parms[$parm_idx];
		   }

		my $output = "> $temp_file_name 2>&1";
		if ($name eq "wmq" && $process eq 'I') {
			$parms = "-s $BASEDIR/$CORE_INSTALLDIR/config -t $log_dir";
			$ENV{'SQLPATH'} = $log_dir . ":" . $ENV{'SQLPATH'};
			$output = "";
			system("touch $temp_file_name");
		} 
		elsif ( $name eq "query_startofweek" && $process eq 'I') {
			$parms = "-t $BASEDIR/$CORE_INSTALLDIR/config";
			if ($silent eq 'Y'){
			   $parms = $parms . " -s " . $responses{START_OF_WEEK};
			}

			$ENV{'SQLPATH'} = "$BASEDIR/$CORE_INSTALLDIR/config:" . $ENV{'SQLPATH'};
			$output = "";
			system("touch $temp_file_name");
		} 

                $name = $name . " " . $parms;
                $return_code =
                  system(
                         "java \-cp $classpath $name -v $process_mode $output"
                        );
                chdir($curr_path);
            }
            else {
                $return_code =
                  system("$exec_path $process_mode > $temp_file_name 2>&1");
            }

            my ($rc) = $return_code / 256;

            my $return_text;
            if ($return_code_text{$rc}) {
              $return_text = $return_code_text{$rc};
            }
            else {
              $return_text = "UNKNOWN ERROR CODE " . $rc;
            }

            my ($short_name, undef) = split(" ",$script_name);
            $short_name = basename($short_name);

            &wm_audit::print_detail( "$short_name ", $return_text, "Y",
                "Y", "Y" );
            open LOG_DIR, "<$temp_file_name"
              or die "Unable to open temporary file : $!";
            while (<LOG_DIR>) {
                chomp;
                &wm_audit::print_detail( "       ", $_, "Y", "N", "$verbosity" );
            }
            &wm_audit::print_detail( "", "", "Y", "Y", "Y");
            close LOG_DIR or die "Unable to close temporary file : $!";
            unlink "$temp_file_name";

            if ( $rc ne 0 ) {
                if ( $rc eq 1 ) {
                    &wm_err::accum_error( "$script_name ", "Warning",
                        "Return Code : $rc" );
                }
                else {
                    &wm_err::accum_error( "$script_name ", "Error",
                        "Return Code : $rc" );
                    if ( $global_switch eq 'Y' ) {
                        if ( $rc gt $global_pre_rc ) {
                            $global_pre_rc = $rc;
                        }
                    }
                    else {
                        &wm_audit::print_section( "Abort Processing ", "Y", "Y",
                            "Y" );
                        &wm_err::dump_errors($verbosity);
                        &wm_audit::close_reports;
                        &archive_logs;
                        exit(1);
                    }
                }
            }
        }
    }
}

sub process_modules_module_routines ( $$$$$ ) {
###################################################################
#                                                                 #
###################################################################

    my ( $name, $install_dir, $filename, $process, $tarball_number ) = @_;

    my ( $DBCONNECT, $BASEDIR, $CORE_INSTALLDIR, $PORTBASE,
        $TMPDIR, $CLASSPATH )
      = &wm_xml_handler::return_core_defaults;

    my ($tree_head) = join "/", $BASEDIR, $CORE_INSTALLDIR;
    my ($appended_path) = "";

    &wm_audit::print_section( "Processing $name Routines ", "Y", "Y", "Y" );
    if ( $name ne "CORE" ) {
        if ( $install_dir =~ /^\./ ) {
            $appended_path = join "/", $tree_head, $install_dir;
        }
        else {
            $appended_path = join "/", $tree_head;
        }
    }
    else {
        $appended_path = join "/", $tree_head;
    }

    my $return_code;
    ($return_code) =
      &process_scripts( $appended_path, "PRE_VALIDATE  ", "N", $process,
        &wm_xml_handler::return_modules_module_prevalidate($tarball_number) );
    ($return_code) =
      &process_scripts( $appended_path, "PRE_INSTALL   ", "N", $process,
        &wm_xml_handler::return_modules_module_preinstall($tarball_number) );
    ($return_code) =
      &process_scripts( $appended_path, "INSTALL       ", "N", $process,
        &wm_xml_handler::return_modules_module_install($tarball_number) );
    ($return_code) =
      &process_scripts( $appended_path, "POST_INSTALL  ", "N", $process,
        &wm_xml_handler::return_modules_module_postinstall($tarball_number) );
    ($return_code) =
      &process_scripts( $appended_path, "POST_VALIDATE ", "N", $process,
        &wm_xml_handler::return_modules_module_postvalidate($tarball_number) );

    CompileSchema() if ($process eq 'U');
}

sub post_processing ( ) {
###################################################################
#                                                                 #
###################################################################
    my ( $DBCONNECT, $BASEDIR, $CORE_INSTALLDIR, $PORTBASE, 
        $TMPDIR, $CLASSPATH )
      = &wm_xml_handler::return_core_defaults;

    my ($post_dir) = join "/", $BASEDIR, $CORE_INSTALLDIR;

    &wm_audit::print_section( "Performing Global POST Routines ", "Y", "Y",
        "Y" );

    my ($return_code);
    ($return_code) =
      &process_scripts( $log_dir, "   POST_VALIDATE", "Y", "",
        &wm_xml_handler::return_global_post_validate );
    ($return_code) =
      &process_scripts( $log_dir, "   POST_INSTALL ", "Y", "",
        &wm_xml_handler::return_global_post_install );

}

sub archive_logs ( ) {
###################################################################
#                                                                 #
###################################################################
    my ( $DBCONNECT, $BASEDIR, $CORE_INSTALLDIR, $PORTBASE, 
        $TMPDIR, $CLASSPATH )
      = &wm_xml_handler::return_core_defaults;

    if ( !$log_dir ) { $log_dir = "/tmp"; }

    my ( $name, @dummy ) = fileparse($log_dir);

    &wm_audit::print_section( "Archiving Log Files ", "Y", "Y", "Y" );

    my $audit_dir = join "/", $BASEDIR, $CORE_INSTALLDIR, "audit", $name;

    mkpath( $audit_dir, 0, 0755 );

    chdir($BASEDIR);

    &wm_audit::print_detail( "Log Files Moved to $audit_dir ", " ", "Y", "Y",
        "Y" );
    copy( "$log_dir/$spec",    "$audit_dir/$spec" );
    copy( "$log_dir/summary.log",  "$audit_dir/summary.log" );
    copy( "$log_dir/detail.log",   "$audit_dir/detail.log" );
    copy( "$log_dir/register_rec", "$audit_dir/register_rec" );

    if ( -e "/var/tmp/wminstall/wminstall.log" ) {
        copy( "/var/tmp/wminstall/wminstall.log", "$audit_dir/wminstall.log" );
        unlink "/var/tmp/wminstall/wminstall.log";
    }

    if ( -e "$TMPDIR/core.build" ) {
        copy( "$TMPDIR/core.build", "$audit_dir/core.build" );
        system("rm -rf $TMPDIR/core.build");
    }
    if ( -e "$TMPDIR/vendor.build" ) {
        copy( "$TMPDIR/vendor.build", "$audit_dir/vendor.build" );
        system("rm -rf $TMPDIR/vendor.build");
    }

    my $unlink_file = join "/", $audit_dir, "../..", "$spec";

    if ( -f $unlink_file ) {
        system("rm -rf $unlink_file");
    }

    if ( -d $log_dir ) {
        if ( $log_dir ne "/tmp" ) {
            system("rm -rf $log_dir");
        }
    }

}

sub usage ( ) {
###################################################################
#                                                                 #
###################################################################

    &wm_audit::print_section( "Valid Options ", "Y", "Y", $verbosity );
    &wm_audit::print_detail( "-d | -db_connect  ", "Oracle Connect String", "Y",
        "Y", $verbosity );
    &wm_audit::print_detail( "-b | -basedir     ",
        "Core base install directory", "Y", "Y", $verbosity );
    &wm_audit::print_detail( "-i | -install_dir ",
        "Core subdirectory (version)", "Y", "Y", $verbosity );
    &wm_audit::print_detail( "-p | -portbase    ",
        "Beginning Prospect Port Number", "Y", "Y", $verbosity );
    &wm_audit::print_detail( "-t | -tmpdir      ",
        "Installer temporary directory", "Y", "Y", $verbosity );
    &wm_audit::print_detail( "-r | -response_file     ",
        "Run in silent mode with the specified response file ", "Y", "Y", $verbosity );
    &wm_audit::print_detail( "-s | -spec     ",
        "Spec file", "Y", "Y", $verbosity );
    &wm_audit::print_detail( "-v | -verbose     ", "Verbose", "Y", "Y",
        $verbosity );
}

sub install_tarballs ( ) {
###################################################################
#                                                                 #
###################################################################

    sleep 5;  #Give any processes watching logs a chance to catch up

    my ( $DBCONNECT, $BASEDIR, $CORE_INSTALLDIR, $PORTBASE,
        $TMPDIR, $CLASSPATH )
      = &wm_xml_handler::return_core_defaults;

    my (@tarball_array) = &wm_xml_handler::return_tarballs;
    my ($tree_head)     = join "/", $BASEDIR;
    my ($return_code)   = 0;
    my ( $name, $install_dir, $filename ) = "";

    if ( !-d $tree_head ) {
        mkpath( $tree_head, 0, 0755 );
    }

    my ($dir_to_make);

    for ( my $x = 0 ; $x <= $#tarball_array ; $x++ ) {
        my ( $name, $install_dir, $filename, $process ) =
          split ( ":", $tarball_array[$x] );
        next if ( $process eq 'S' );
        &wm_audit::print_section( "Extract Tarballs ", "Y", "Y", $verbosity );
        &wm_audit::print_detail( "Extract ", "$filename", "Y", "Y",
            "$verbosity" );
        my ($returned_register_record) =
          &wm_utilities::create_register_files($x);
        my ($register_record) = join "/", $TMPDIR, "register_rec";

        open REGISTER_FILE, ">$register_record"
          or die "Unable to open register_rec for write : $!";
        print REGISTER_FILE "$returned_register_record";
        close REGISTER_FILE or die "Unable to close register_rec : $!";

        if ( $name eq "CORE" ) {
            $dir_to_make = join "/", $tree_head, $CORE_INSTALLDIR;
            if ( !-d $dir_to_make ) {
                mkpath( $dir_to_make, 0, 0755 );
            }
        }
        else {
            if ( $install_dir =~ /^\./ ) {
                $dir_to_make = join "/", $tree_head, $CORE_INSTALLDIR,
                  $install_dir;
                mkpath( $dir_to_make, 0, 0755 );
            }
            else {
                $dir_to_make = join "/", $tree_head, $CORE_INSTALLDIR, "vendor",
                  $install_dir;
            }
        }

        if ( $name ne 'CORE' ) {
            if ( $install_dir =~ /^\./ ) {
                $return_code =
                  &wm_utilities::untar( "$start_dir", "$start_dir/$filename",
                    $tree_head . "/" . $CORE_INSTALLDIR . "/" . $install_dir,
                    "N", "" );
            }
            else {
                $return_code =
                  &wm_utilities::untar( "$start_dir", "$start_dir/$filename",
                    $tree_head . "/" . $CORE_INSTALLDIR, "N", "" );
            }
        }
        else {
            $return_code =
              &wm_utilities::untar( "$start_dir", "$start_dir/$filename",
                $tree_head . "/" . $CORE_INSTALLDIR, "N", "" );
        }

        if ( $process eq 'U' ) {
            &wm_utilities::run_register( "START", $register_record );
        }
        my ($return_code) =
          &process_modules_module_routines( $name, $install_dir, $filename,
            $process, $x );
        if ( $process eq 'I' ) {
            &wm_utilities::run_register( "START", $register_record );
        }
        &wm_utilities::run_register( "COMPLETE", $register_record );
        unlink "$register_record";

    }

    my $unlink_dir = join "/", $log_dir, "$name";

    if ( -d $name ) {
        my $unlink_code = system("rm -rf $unlink_dir");
    }

    return ($return_code);

}

sub global_pre_routine_processing ( ) {
###################################################################
#                                                                 #
###################################################################

    &wm_audit::print_section( "Performing Global PRE Routines ", "Y", "Y",
        "Y" );

    my $return_code;

    ($return_code) =
      &process_scripts( $log_dir, "   PRE_VALIDATE", "Y", "",
        &wm_xml_handler::return_global_pre_validate );

    if ( $preview eq "Y" ) {
        return;
    }

    ($return_code) =
      &process_scripts( $log_dir, "   PRE_INSTALL ", "Y", "",
        &wm_xml_handler::return_global_pre_install );

}

sub test_errors ( ) {
###################################################################
#                                                                 #
###################################################################

    my ( $error_count, $warning_count ) = &wm_err::return_error_counts;

    if ( $error_count gt 0 ) {
        &wm_audit::print_section( "Abort Processing ", "Y", "Y", "Y" );
        &wm_err::dump_errors($verbosity);
        &wm_audit::close_reports;
        if ( $preview ne 'Y' ) {
             &archive_logs;
	   }
        exit(-1);
    }
}

sub validate_relative_paths ( ) {
###################################################################
#                                                                 #
###################################################################
    my ( $DBCONNECT, $BASEDIR, $CORE_INSTALLDIR, $PORTBASE,
        $TMPDIR, $CLASSPATH )
      = &wm_xml_handler::return_core_defaults;

    my (@vendor_install_dir) = &wm_xml_handler::return_vendor_modules_module;

    if ( $CORE_INSTALLDIR =~ /^\// ) {
        &wm_err::accum_error(
            "CORE_INSTALLDIR should be a relative path to $BASEDIR", "Error",
            "$CORE_INSTALLDIR must not begin with a '/'"
        );
    }

    for ( my $x = 0 ; $x <= $#vendor_install_dir ; $x++ ) {
        if ( $vendor_install_dir[$x] =~ /^\// ) {
            &wm_err::accum_error(
"MODULE_INSTALLDIR should be a relative path to $BASEDIR/$CORE_INSTALLDIR",
                "Error", "$vendor_install_dir[$x] must not begin with a '/'" );
        }
    }
}

sub validate_run_scripts ( $$ ) {
###################################################################
#                                                                 #
###################################################################

    my ( $start_dir, $log_dir ) = @_;

    my (@tarball_array) = &wm_xml_handler::return_tarballs;

    &wm_audit::print_section( "Validating Global Routines ", "Y", "Y",
        "$verbosity" );

    &loop_thru_array(
        "PRE_VALIDATE   ",
        &wm_xml_handler::return_global_pre_validate
    );
    &loop_thru_array(
        "PRE_INSTALL    ",
        &wm_xml_handler::return_global_pre_install
    );
    &loop_thru_array(
        "POST_INSTALL   ",
        &wm_xml_handler::return_global_post_install
    );
    &loop_thru_array(
        "POST_VALIDATE  ",
        &wm_xml_handler::return_global_post_validate
    );

    for ( my $x = 0 ; $x <= $#tarball_array ; $x++ ) {
        my ( $name, $install_dir, $filename, $process ) =
          split ( ":", $tarball_array[$x] );
        &wm_audit::print_section( "Verifying $name Routines - $filename ", "Y", "Y",
            $verbosity );
        my $appended_path;
        if ( $name eq 'CORE' ) {
            $appended_path = join "/", $log_dir, $install_dir;
        }
        else {
            $appended_path = join "/", $log_dir, $name;
        }

        &validate_routines( "PRE_VALIDATE  ", $filename,
            &wm_xml_handler::return_modules_module_prevalidate($x) );
        &validate_routines( "PRE_INSTALL   ", $filename,
            &wm_xml_handler::return_modules_module_preinstall($x) );
        &validate_routines( "INSTALL       ", $filename,
            &wm_xml_handler::return_modules_module_install($x) );
        &validate_routines( "POST_INSTALL  ", $filename,
            &wm_xml_handler::return_modules_module_postinstall($x) );
        &validate_routines( "POST_VALIDATE ", $filename,
            &wm_xml_handler::return_modules_module_postvalidate($x) );
    }

}


sub spec_file_archive ( ) {
###################################################################
#  spec_file_archive :  copy core.spec and modules.spec to the    #
#       log_dir directory for future reference.                   #
###################################################################

    if ($spec_exists) {
        &wm_audit::print_section( "Copy $spec ", "Y", "Y", $verbosity );
        if ( copy( "$spec_file", "$log_dir" ) ) {
            &wm_audit::print_detail( "Copy   ", "Successful", "Y", "Y",
                $verbosity );
        }
        else {
            &wm_err::accum_error( "$spec copy failed ", "Error", "1" );
            &wm_audit::print_detail( "Copy ", "Failed", "Y", "Y", $verbosity );
        }
    }
}

sub parse_format ( $ ) {
###################################################################
#                                                                 #
###################################################################

    my ($passed_var) = @_;

    if ( ($passed_var) && ( $passed_var !~ /^-/ ) ) {
        return (1);
    }
    else {
        return (0);    # this is an error condition
    }
}

sub validate_defaults ( ) {
###################################################################
#                                                                 #
###################################################################
    my ( $DBCONNECT, $BASEDIR, $CORE_INSTALLDIR, $PORTBASE,
        $TMPDIR, $CLASSPATH )
      = &wm_xml_handler::return_core_defaults;

    my ($valid_db_connect)      = 0;
    my ($valid_basedir)         = 0;
    my ($valid_core_installdir) = 0;
    my ($valid_portbase)        = 0;
    my ($valid_tmpdir)          = 0;
    my ($valid_classpath)       = 0;

    if ( &parse_format($DBCONNECT) )       { $valid_db_connect      = 1; }
    if ( &parse_format($BASEDIR) )         { $valid_basedir         = 1; }
    if ( &parse_format($CORE_INSTALLDIR) ) { $valid_core_installdir = 1; }
    if ( &parse_format($PORTBASE) )        { $valid_portbase        = 1; }
    if ( &parse_format($TMPDIR) )         { 
         if ( $TMPDIR =~ /^\// ) {
             $valid_tmpdir  = 1;
         }
         else
	   {
             $valid_tmpdir = 0;
	   }
       }
    if ( &parse_format($CLASSPATH) )       { $valid_classpath       = 1; }

    if ( ( $valid_db_connect == 0 )
        || ( $valid_basedir == 0 )
        || ( $valid_core_installdir == 0 )
        || ( $valid_portbase == 0 )
        || ( $valid_tmpdir == 0 )
        || ( $valid_classpath == 0 ) )
    {
        &wm_audit::print_section( "Validating Defaults ", "Y", "Y",
            $verbosity );
        if ( $valid_db_connect == 0 ) {
            &wm_err::accum_error( "DB_CONNECT String($DBCONNECT) is invalid ",
                "Error", "2" );
            &wm_audit::print_detail(
                "DB_CONNECT String($DBCONNECT) is invalid ", "Error", "Y", "Y",
                $verbosity );
        }
        if ( $valid_basedir == 0 ) {
            &wm_err::accum_error( "BASEDIR ($BASEDIR) is invalid ", "Error",
                "2" );
            &wm_audit::print_detail( "BASEDIR ($BASEDIR) is invalid ", "Error",
                "Y", "Y", $verbosity );
        }
        if ( $valid_core_installdir == 0 ) {
            &wm_err::accum_error(
                "CORE_INSTALLDIR ($CORE_INSTALLDIR) is invalid ", "Error",
                "2" );
            &wm_audit::print_detail(
                "CORE_INSTALLDIR ($CORE_INSTALLDIR) is invalid ", "Error", "Y",
                "Y", $verbosity );
        }
        if ( $valid_portbase == 0 ) {
            &wm_err::accum_error( "PORTBASE ($PORTBASE) is invalid ", "Error",
                "2" );
            &wm_audit::print_detail( "PORTBASE ($PORTBASE) is invalid ",
                "Error", "Y", "Y", $verbosity );
        }
        if ( $valid_tmpdir == 0 ) {
            &wm_err::accum_error( "TMPDIR ($TMPDIR) is invalid ", "Error",
                "2" );
            &wm_audit::print_detail( "TMPDIR ($TMPDIR) is invalid ", "Error",
                "Y", "Y", $verbosity );
        }
        if ( $valid_classpath == 0 ) {
            &wm_err::accum_error( "CLASSPATH ($CLASSPATH) is invalid ", "Error",
                "2" );
            &wm_audit::print_detail( "CLASSPATH ($CLASSPATH) is invalid ",
                "Error", "Y", "Y", $verbosity );
        }
    }

####
    # We don't call test_errors here because we may be missing basedir and core_installdir
###
    my ( $error_count, $warning_count ) = &wm_err::return_error_counts;

    if ( $error_count gt 0 ) {
        &wm_audit::print_section( "Abort Processing ", "Y", "Y", "Y" );
        &wm_err::dump_errors($verbosity);
        &wm_audit::close_reports;
        exit(-1);
    }
}

sub update_defaults ( ) {
###################################################################
#                                                                 #
###################################################################

    my ( $DBCONNECT, $BASEDIR, $CORE_INSTALLDIR, $PORTBASE,
        $TMPDIR, $CLASSPATH )
      = &wm_xml_handler::return_core_defaults;

    my (
        $Module_DBCONNECT, $Module_BASEDIR,     $Module_INSTALLDIR,
        $Module_PORTBASE,  $Module_TMPDIR,
        $Module_CLASSPATH
      )
      = &wm_xml_handler::return_module_overrides;

    if ($cl_db_connect) {
        $DBCONNECT = $cl_db_connect;
    }
    else {
        if ($Module_DBCONNECT) {
            $DBCONNECT = $Module_DBCONNECT;
        }
    }

    if ($cl_basedir) {
        $BASEDIR = $cl_basedir;
    }
    else {
        if ($Module_BASEDIR) {
            $BASEDIR = $Module_BASEDIR;
        }
    }

    if ($cl_install_dir) {
        $CORE_INSTALLDIR = $cl_install_dir;
    }
    else {
        if ($Module_INSTALLDIR) {
            $CORE_INSTALLDIR = $Module_INSTALLDIR;
        }
    }

    if ($cl_port_base) {
        $PORTBASE = $cl_port_base;
    }
    else {
        if ($Module_PORTBASE) {
            $PORTBASE = $Module_PORTBASE;
        }
    }

    if ($cl_tmpdir) {
        $TMPDIR = $cl_tmpdir;
    }
    else {
        if ($Module_TMPDIR) {
            $TMPDIR = $Module_TMPDIR;
        }
    }

    if ($cl_classpath) {
        $CLASSPATH = $cl_classpath;
    }
    else {
        if ($Module_CLASSPATH) {
            $CLASSPATH        = $Module_CLASSPATH;
            $global_classpath = $CLASSPATH;
        }
    }

    &wm_xml_handler::update_defaults( $DBCONNECT, $BASEDIR, $CORE_INSTALLDIR,
        $PORTBASE, $TMPDIR, $CLASSPATH );

    $ENV{'CLASSPATH'} = $CLASSPATH;

    &wm_audit::print_section( "Core Composite Defaults ", "Y", "Y", "Y" );
    &wm_audit::print_detail( "DBCONNECT      ", $DBCONNECT, "Y", "Y", "Y" );
    &wm_audit::print_detail( "SYSPASSWD  ", "<Set>",    "Y", "Y", "Y" );
    &wm_audit::print_detail( "BASEDIR        ", $BASEDIR,   "Y", "Y", "Y" );
    &wm_audit::print_detail( "CORE_INSTALLDIR", $CORE_INSTALLDIR, "Y", "Y",
        "Y" );
    &wm_audit::print_detail( "PORTBASE       ", $PORTBASE,    "Y", "Y", "Y" );
    &wm_audit::print_detail( "TMPDIR         ", $TMPDIR,      "Y", "Y", "Y" );
    &wm_audit::print_detail( "CLASSPATH      ", $CLASSPATH,   "Y", "Y", "Y" );
}

sub print_passed_parms ( ) {
###################################################################
#                                                                 #
###################################################################

    &wm_audit::print_section( "Passed Parms ", "Y", "Y", "Y" );
    if ($cl_db_connect) {
        &wm_audit::print_detail( "db_connect     ", $cl_db_connect, "Y", "Y",
            "Y" );
    }
    else {
        &wm_audit::print_detail( "db_connect     ", "<No Override>", "Y", "Y",
            $verbosity );
    }

    if ($cl_basedir) {
        &wm_audit::print_detail( "basedir        ", $cl_basedir, "Y", "Y",
            "Y" );
    }
    else {
        &wm_audit::print_detail( "basedir        ", "<No Override>", "Y", "Y",
            $verbosity );
    }

    if ($cl_install_dir) {
        &wm_audit::print_detail( "install_dir    ", $cl_install_dir, "Y", "Y",
            "Y" );
    }
    else {
        &wm_audit::print_detail( "install_dir    ", "<No Override>", "Y", "Y",
            $verbosity );
    }

    if ($cl_port_base) {
        &wm_audit::print_detail( "portbase       ", $cl_port_base, "Y", "Y",
            "Y" );
    }
    else {
        &wm_audit::print_detail( "portbase       ", "<No Override>", "Y", "Y",
            $verbosity );
    }

    if ($cl_tmpdir) {
        &wm_audit::print_detail( "tmpdir         ", $cl_tmpdir, "Y", "Y", "Y" );
    }
    else {
        &wm_audit::print_detail( "tmpdir         ", "<No Override>", "Y", "Y",
            $verbosity );
    }

    if ($cl_classpath) {
        &wm_audit::print_detail( "classpath      ", $cl_classpath, "Y", "Y",
            "Y" );
    }
    else {
        &wm_audit::print_detail( "classpath      ", "<No Override>", "Y", "Y",
            $verbosity );
    }

    if ($cl_spec) {
        &wm_audit::print_detail( "spec      ", $cl_spec, "Y", "Y",
            "Y" );
    }
    else {
        &wm_audit::print_detail( "spec      ", "<No Override>", "Y", "Y",
            $verbosity );
    }

    if ( $preview eq "Y" ) {
        &wm_audit::print_detail( "Process Mode    ", "PREVIEW", "Y", "Y", "Y" );
    }
    else {
        &wm_audit::print_detail( "Process Mode    ", "INSTALL", "Y", "Y", "Y" );
    }
}

sub print_module_overrides ( ) {
###################################################################
#                                                                 #
###################################################################

    my ( $DBCONNECT, $BASEDIR, $CORE_INSTALLDIR, $PORTBASE, 
        $TMPDIR, $CLASSPATH )
      = &wm_xml_handler::return_module_overrides;

    &wm_audit::print_section( "Module Overrides", "Y", "Y", "Y" );

    if ($DBCONNECT) {
        &wm_audit::print_detail( "DBCONNECT       ", $DBCONNECT, "Y", "Y",
            "Y" );
    }
    else {
        &wm_audit::print_detail( "DBCONNECT       ", "<No Override>", "Y", "Y",
            "Y" );
    }

    if ($BASEDIR) {
        &wm_audit::print_detail( "BASEDIR         ", $BASEDIR, "Y", "Y", "Y" );
    }
    else {
        &wm_audit::print_detail( "BASEDIR         ", "<No Override>", "Y", "Y",
            "Y" );
    }

    if ($CORE_INSTALLDIR) {
        &wm_audit::print_detail( "CORE_INSTALLDIR ", $CORE_INSTALLDIR, "Y", "Y",
            "Y" );
    }
    else {
        &wm_audit::print_detail( "CORE_INSTALLDIR ", "<No Override>", "Y", "Y",
            "Y" );
    }

    if ($PORTBASE) {
        &wm_audit::print_detail( "PORTBASE        ", $PORTBASE, "Y", "Y", "Y" );
    }
    else {
        &wm_audit::print_detail( "PORTBASE        ", "<No Override>", "Y", "Y",
            "Y" );
    }

    if ($TMPDIR) {
        &wm_audit::print_detail( "TMPDIR          ", $TMPDIR, "Y", "Y", "Y" );
    }
    else {
        &wm_audit::print_detail( "TMPDIR          ", "<No Override>", "Y", "Y",
            "Y" );
    }

    if ($CLASSPATH) {
        &wm_audit::print_detail( "CLASSPATH       ", $CLASSPATH, "Y", "Y",
            "Y" );
    }
    else {
        &wm_audit::print_detail( "CLASSPATH       ", "<No Override>", "Y", "Y",
            "Y" );
    }

}

sub print_core_defaults ( ) {
###################################################################
#                                                                 #
###################################################################

    my ( $DBCONNECT, $BASEDIR, $CORE_INSTALLDIR, $PORTBASE,
        $TMPDIR, $CLASSPATH )
      = &wm_xml_handler::return_core_defaults;

    &wm_audit::print_section( "Core Defaults   ", "Y", "Y", "Y" );
    &wm_audit::print_detail( "DBCONNECT       ", $DBCONNECT, "Y", "Y", "Y" );
    &wm_audit::print_detail( "BASEDIR         ", $BASEDIR,   "Y", "Y", "Y" );
    &wm_audit::print_detail( "CORE_INSTALLDIR ", $CORE_INSTALLDIR, "Y", "Y",
        "Y" );
    &wm_audit::print_detail( "PORTBASE        ", $PORTBASE,    "Y", "Y", "Y" );
    &wm_audit::print_detail( "TMPDIR          ", $TMPDIR,      "Y", "Y", "Y" );
    &wm_audit::print_detail( "CLASSPATH       ", $CLASSPATH,   "Y", "Y", "Y" );

}

sub report_tarball_state ( ) {
###################################################################
# report_tarball_state : log tarball activity to the log files    #
###################################################################

    &wm_audit::print_section( "Installer Self-Audit ", "Y", "Y", $verbosity );

    if ($spec_exists) {
        &wm_audit::print_detail( "$spec", "Exists", "Y", "Y", $verbosity );
    }
    else {
        &wm_err::accum_error( "$spec is missing ", "Warning", "1" );
        &wm_audit::print_detail( "$spec is missing ", "Warning", "Y", "Y",
            $verbosity );
    }

    my (@tarball_array) = &wm_xml_handler::return_tarballs;

    for ( my $i = 0 ; $i <= $#tarball_array ; $i++ ) {
        my ( $comp_name, $dummy, $tarfile_name, $process ) =
          split ( /:/, $tarball_array[$i] );
        if ( -f $tarfile_name ) {
            &wm_audit::print_detail( "$comp_name - $tarfile_name", "Exists",
                "Y", "Y", $verbosity );
        }
        else {
            &wm_err::accum_error( "$comp_name - $tarfile_name missing ",
                "Error", "1" );
            &wm_audit::print_detail( "$comp_name - $tarfile_name missing ",
                "Error", "Y", "Y", $verbosity );
        }
    }
}

sub license_agreement ( ) {
###################################################################
#                                                                 #
###################################################################
  #Check the license agreement in case of silent install
  if ($silent eq 'Y') {
    die "\n####\n"
      . "#### License Acceptance in response file is not true\n"
      . "####\n"
      if ($responses{'LICENSE_ACCEPTANCE'} ne 'TRUE');
      return;
  }

  #Check for license agreement text
  my $lic_text="license.txt";
  die "\n####\n" .
      "#### ERROR: License text file $lic_text not found\n" .
      "####\n"
        if (! -f $lic_text);

  #Page it
  my $pager = $ENV{PAGER} || "more";
  $pager = "/bin/less -P \"(f forward, b back, q quit) \"" if (-f "/bin/less"); #Really prefer less
  system("$pager $lic_text");

  #Confirm
  print "Accept licence agreement? [yes|NO] ";
  $| = 1;
  $_ = <STDIN>;
  chomp;
  if (!/^yes$/i) {
    die "\nLicence agreement not accepted. Aborting.\n";
  }
}

sub determine_log_dir ( ) {
###################################################################
#                                                                 #
###################################################################

    my ( $d1, $d2, $d3, $d4, $tmpdir, $classpath ) =
      &wm_xml_handler::return_core_defaults;

    my ( $log_dir_valid ) = 0;

    if ($cl_tmpdir) {
      if ( $cl_tmpdir =~ /^\// ) {
	if (! -w $cl_tmpdir) {
	  print STDERR "####\n";
	  print STDERR "#### The directory passed after the -t option on the commandline \n";
	  print STDERR "#### ($cl_tmpdir) is not writable\n";
	  print STDERR "#### Please correct and then rerun\n";
	  print STDERR "####\n";
	}
	else {
	  $log_dir = &wm_utilities::create_tmp_dir($cl_tmpdir);
	  $log_dir_valid = 1;
	} # end of if
      }
      else {
	print STDERR "####\n";
	print STDERR "#### The directory passed after the -t option on the commandline \n";
	print STDERR "#### ($cl_tmpdir) is invalid\n";
	print STDERR "#### The directory must be an absolute path, not a relative path\n";
	print STDERR "#### Please correct and then rerun\n";
	print STDERR "####\n";
      } # end of if
    }
    else {
      if ($tmpdir) {
	if ( $tmpdir =~ /^\// ) {
	  if (! -w $tmpdir) {
	    print STDERR "####\n";
	    print STDERR "#### The TMPDIR value in the $spec ($tmpdir) is not writable\n";
	    print STDERR "#### Please correct and then rerun\n";
	    print STDERR "####\n";
	  }
	  else {
            $log_dir = &wm_utilities::create_tmp_dir($tmpdir);
            $log_dir_valid = 1;
	  } # end of if
	}
	else {
	  print STDERR "####\n";
	  print STDERR "#### The TMPDIR value in the $spec ($tmpdir) is invalid\n";
	  print STDERR "#### The directory must be an absolute path, not a relative path\n";
	  print STDERR "#### Please correct and then rerun\n";
	  print STDERR "####\n";
	} # end of if
      }
      else {
	$log_dir = &wm_utilities::create_tmp_dir("/tmp");
	$log_dir_valid = '1';
      } # end of if
    } # end of if
    
    if ( $log_dir_valid ) {
      &wm_audit::start_audit_report($log_dir);
    }
    else {
      die;
    } # ene of if
  }

sub test_silent_install ( ) {
###################################################################
#  test_silent_install :  Checks if it is a silent install        #
#                                                                 #
###################################################################
  if ($silent eq 'Y') {
    die( "ERROR: response file $response_file not found")
    if !( -f $response_file );

    # parse the file. get every line with 'key = value'
    # skip every comment line starting with '#'
    open(RES,"< $response_file") || die ("$response_file can't be opened : error $!");
    while(my $line = <RES>){
      chomp($line);
      if ($line !~ /^[ #]*#/ && $line =~ /\s*(\w+)\s*=\s*([\w\/]+)\s*/) {
        $responses{uc($1)} = uc($2);
      }
    }
    close(RES);
  }
}

sub test_spec_files ( ) {
###################################################################
#  test_spec_files :  Checks that there is spec.xml               #
#  on the installation media.                                     #
###################################################################

    my $return_code;
    my $test_spec;
    
    if ($spec_dir) {
        $test_spec = join( "/", $spec_dir, "$spec_file" );
    }
    else {
        $test_spec = join( "/", $start_dir, "$spec_file" );
    }
        
    if ( -f $test_spec ){
        $spec_exists = "1";
    }
    else {
        $spec_exists = "0";    # spec.xml not found
    }

    if ($spec_exists){
        $spec_file = $test_spec;
    }
    else {
        $spec_file = "";
    }
}

sub abort_processing ( $ ) {
###################################################################
#                                                                 #
###################################################################

    my ($print_return_code) = @_;

    &wm_audit::print_detail( "System State Unknown - $print_return_code",
        "Error", "Y", "Y", "Y" );
    &wm_err::accum_error(
        "System State Unknown - $print_return_code", "Error",
        "Processing aborted"
    );
    &wm_err::dump_errors($verbosity);
    &wm_audit::close_reports;
    &archive_logs;
    exit;
}

#
# CompileSchema() routine
#
sub CompileSchema {
  my $option;
  my ( $DBCONNECT, $BASEDIR, $CORE_INSTALLDIR, $PORTBASE, $TMPDIR, $CLASSPATH )
    = &wm_xml_handler::return_core_defaults;
  
  &wm_audit::print_section( "Compile Schema ", "Y", "Y", "$verbosity" );
  
  my $ORACLE_VERS=`basename $ENV{'ORACLE_HOME'} | tr -dc '[0-9]'`;

  if ($ORACLE_VERS =~ m/920|817/) {
    $option = ',false';
  }
  elsif ($ORACLE_VERS =~ m/816|81/) {
    $option=" ";
  } # end of if
  
  open(SQL_CMD, "|sqlplus -s /nolog \n");
  print SQL_CMD "whenever sqlerror exit sql.sqlcode ;\n";
  print SQL_CMD "connect $DBCONNECT \n";
  print SQL_CMD "exec dbms_utility.compile_schema ( user $option) ;\n";
  print SQL_CMD "exit; \n";
  close(SQL_CMD);

  if ($?) {
    &wm_err::accum_error( "Compile Schema failed ", "Error", "1" );
    &wm_err::dump_errors($verbosity);
    &wm_audit::close_reports;
    &archive_logs;
    exit;
  } 
} # end of CompileSchema()

sub main {
###################################################################
#                          The APP                                #
###################################################################
    &test_silent_install;
    
    if ($cl_spec) {
      $spec_dir = dirname($cl_spec);
      $spec_file = basename($cl_spec);
    }

    $start_dir = cwd();

    if ( !-d '/var/tmp/wminstall' ) {
        mkpath( '/var/tmp/wminstall', 0, 0755 );
    }

    &test_spec_files;

###
###  Parse the two xml files 
###

    &wm_xml_handler::parse_xml_files($spec_file);

    &determine_log_dir;

    license_agreement();


    &report_tarball_state;

    &print_core_defaults;
    &print_module_overrides;
    &print_passed_parms;
    &update_defaults;
    &validate_defaults;

    if ( $#ARGV ge 0 ) {
        &wm_audit::print_section( "Validating Extra Command Line Options ", "Y",
            "Y", "Y" );
        &wm_audit::print_detail( "Invalid Option - @ARGV ", "Error", "Y", "Y",
            "Y" );
        &usage;
        &test_errors;
        exit;
    }

    chdir($start_dir)
      or die
      "Unable to change back to the starting directory ($start_dir) : $!";

    &spec_file_archive;

    chdir($start_dir)
      or die
      "Unable to change back to the starting directory ($start_dir) : $!";


###
    # Validate that the extracted installation directories actually contain the scripts
    # noted in the spec files.
###

    &validate_run_scripts( $start_dir, $log_dir );

    &validate_relative_paths;

    &test_errors;

###
    #    Dependency checking needs to go in here and adjust the various routines accordingly
###

    &wm_runtype::evaluate_prerequisites($verbosity);
   
    &test_errors;

    &print_composite_routines;

    &global_pre_routine_processing;

    if ( $preview eq "Y" ) {
        &wm_audit::print_section( "Preview option selected ", "Y", "Y", "Y" );
        &wm_audit::print_detail( "Shutting down after preview ", "Warning", "Y",
            "Y", "Y" );
        &wm_err::accum_error( "Shutting down", "Warning",
            "Preview option selected" );
        if ( -f "/tmp/$spec" ) {
          unlink "/tmp/$spec";
        }
        if ( -f '/tmp/process.txt' ) {
            unlink '/tmp/process.txt';
        }
        &wm_err::dump_errors($verbosity);
        &wm_audit::close_reports;
        if ( -d $log_dir ) {
            chdir($start_dir);
            system("rm -rf $log_dir");
        }
        exit;
    }

    &test_errors;

    my $install_tarballs_return = &install_tarballs;

    if ( ( $install_tarballs_return lt 0 )
        || ( $install_tarballs_return gt 1 ) )
    {
        &wm_err::accum_error( "Install failure ( untar ) ", "Error",
            "Processing aborted" );
        &wm_err::dump_errors($verbosity);
        &wm_audit::close_reports;
        &archive_logs;
        exit;

    }

    &post_processing;

    &wm_err::dump_errors($verbosity);

    &wm_utilities::clean_up_files;
    &wm_audit::close_reports;

    &archive_logs;

}

sub run {
###################################################################
#                                                                 #
###################################################################

#### Turn on the pass_through option which will force all unrecognized
#### options into ARGV

    Getopt::Long::Configure(qw(pass_through));

    GetOptions(
        "db_connect=s"   => \$cl_db_connect,
        "basedir=s"      => \$cl_basedir,
        "install_dir=s"  => \$cl_install_dir,
        "portbase=s"     => \$cl_port_base,
        "tmpdir=s"       => \$cl_tmpdir,
        "classpath=s"    => \$cl_classpath,
        "spec=s"        => \$cl_spec,
        "response_file=s"=> \$response_file,
        "preview"        => \$preview,
        "verbose"        => \$verbose
    );

    if ($verbose) { $verbosity = "Y"; }
    else { $verbosity = "N"; }

    if ($preview) { $preview = "Y"; }
    else { $preview = "N"; }
	if ($response_file) { $silent = "Y"; }
	else { $silent = "N"; }
	
	if ($cl_spec){
		$spec = $cl_spec;
	}
	($spec_file) = @_;
	&main;
}

1;
