package wm_utilities;

###################################################################
#
# Licensed Materials - Property of IBM
# 5724-T05,5724-T06
#
# (C) Copyright IBM Corp. 2003,2008. All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
###################################################################
# Script   : wm_utilities                                         #
# Function : Miscellaneous standalone utility functions           #
###################################################################
# (#) ./install/lib/wm_utilities.pm ; BL_prospect_installer_8.0.6.0.13_20081104  181901

###################################################################
#              Standard/Supplied Perl Modules                     #
###################################################################
use strict;
use Cwd;
use Env;
use File::Basename;
use File::Copy;
use File::Path;

###################################################################
#                      Watchmark Perl Modules                     #
###################################################################
use wm_audit;
use wm_xml_handler;

###################################################################
#                     Global Vars                                 #
###################################################################

my $date            = "";
my $curr_time       = "";
my $date_time       = "";
my $DBCONNECT       = "";
my $BASEDIR         = "";
my $CORE_INSTALLDIR = "";
my $PORTBASE        = "";
my $TMPDIR          = "";

sub get_date_time ( ) {
########################################################################
# Get the current date and time for the audit report records.          #
########################################################################

     my ( $sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $isdat ) =
      localtime(time);

    if ( length($hour) < 2 ) { $hour = "0" . $hour; }
    if ( length($min) < 2 )  { $min  = "0" . $min; }
    if ( length($sec) < 2 )  { $sec  = "0" . $sec; }

    $curr_time = $hour . ":" . $min . ":" . $sec;

    if ( length($mon) < 2 )  { $mon  = "0" . $mon; }
    if ( length($mday) < 2 ) { $mday = "0" . $mday; }

    $date = join "_", ( $year + 1900 ), ( ++$mon ), ($mday);

    $curr_time = $hour . ":" . $min . ":" . $sec;

    return ( $date, $curr_time );

}    ### end of get_date_time

sub get_temp_file_name ($) {
########################################################################
# Create and return unique temp file name                              #
########################################################################
     my $tmpdir = shift;
     my ( $sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $isdat ) =
          localtime(time);

     if ( length($hour) < 2 ) { $hour = "0" . $hour; }
     if ( length($min) < 2 )  { $min  = "0" . $min; }
     if ( length($sec) < 2 )  { $sec  = "0" . $sec; }

     if ( length($mon) < 2 )  { $mon  = "0" . $mon; }
     if ( length($mday) < 2 ) { $mday = "0" . $mday; }

     $date = join "_", ( $year + 1900 ), ( ++$mon ), ($mday);

     $curr_time = $hour . "_" . $min . "_" . $sec;
     my $fn = join("/",$tmpdir,join "_",$date,$curr_time);

     #Make sure file is unique
     my $n = 0;
     my $testfile = $fn;
     while (-e $testfile) {
       $testfile .= "." . ++$n;
     }

    return ($testfile);

}    ### end of get_tmp_file_name


sub create_tmp_dir ( $ ){
########################################################################
# Create the TMPDIR as defined in the core.spec file                   #
########################################################################
    my ($TMPDIR) = @_;

    if ( !$TMPDIR ) {
        $TMPDIR = '/tmp';
    }

    if ( !-d $TMPDIR ) {
        my $return_code = mkpath( $TMPDIR, 0, 0755 );
    }

    &get_date_time;

    $date_time = join "_", $date, $curr_time, $$;
    my $dir_name = join "/", $TMPDIR, $date_time;
    $dir_name =~ s/:/_/g;

    my $return_code = mkpath( $dir_name, 0, 0755 );

    return ($dir_name);

}

##########################
#Find path to gtar; complain if we can't find it
##########################
sub find_gtar($) {
  my $verbosity = shift;
  my $gtar = "gtar";
  my $gtpath = "";

  foreach my $path (split(/:/,$ENV{'PATH'})) {
    my $try = "$path/$gtar";
    if (-f $try) {
      $gtpath = $try;
      last;
    }
  }

  if ( $gtpath eq "" ) {
    &wm_audit::print_detail( "$gtar ", "Missing in PATH", "Y", "Y",
                             $verbosity );
    &wm_err::accum_error( "$gtar ", "Error",
                          "Missing in PATH" );
    return (2);
  }
  else {
    return $gtpath;
  }
}

sub untar ( $$$$@ ) {
########################################################################
# untar one or more files from a given tarball to /tmp                 #
########################################################################
    my ( $start_dir, $tarball, $target_dir, $verbosity, @files ) = @_;

    my ($return_code) = 0;

    my ($gtar) = find_gtar($verbosity);
    if ($gtar == 2) {
      return(2);
    }

    $return_code = chdir($target_dir);

    system("chmod -R 755 $target_dir > /dev/null 2>&1");

    for ( my $x = 0 ; $x <= $#files ; $x++ ) {
        $return_code =
          system("$gtar -xzvf $tarball $files[$x] > /tmp/untar.log 2>&1");
    }
    open TAR_LOG, "</tmp/untar.log" or die "Unable to open /tmp/untar.log :$!";

    while (<TAR_LOG>) {
        chomp;
        &wm_audit::print_detail( "                 ", $_, "Y", "N",
            $verbosity );
    }
    close TAR_LOG or die "Unable to close /tmp/untar.log : $!";

    return ( $return_code / 256 );
}


sub create_register_files ( $ ){
###################################################################
#                                                                 #
###################################################################
    my ( $tarball_number ) = @_;

    my ($process_literal) = "";

    my (
        $Core_Default_DBCONNECT,       $Core_Default_BASEDIR,
        $Core_Default_CORE_INSTALLDIR, $Core_Default_PORTBASE,
        $Core_Default_TMPDIR
      )
      = &wm_xml_handler::return_core_defaults;

    my ( $Spec_type, $Module_NAME, $Module_PLATFORM, $Module_VERSION,
            $Process_type, @dummy )
           = &wm_xml_handler::return_prerequisite_versions($tarball_number);

    my ($build_number)   = $Module_VERSION;

    my (@work_build_number) = split /\./, $build_number;
    
    my $revision = $work_build_number[0];
    for ( my $x = 1;$x < $#work_build_number;$x++) {
         $revision = $revision . "." . $work_build_number[$x] ;
       }

    my ($register_build) = $work_build_number[$#work_build_number];

    if ( $Process_type eq "I" ) { 
         $process_literal = "INSTALL";
    }
    else {
        if ( $Process_type eq "U" ) {
            $process_literal = "UPGRADE";
        }
    }

    if ( $Module_NAME ne 'CORE' ) { $Module_NAME = 'VENDOR'; }

    my ( $register_record ) = join "\t",$process_literal,$Module_NAME, $Module_PLATFORM,$revision,$register_build;

    return ( $register_record );
}


sub run_register ( $$ ){
###################################################################
#                                                                 #
###################################################################
    my ( $process_literal, $file_location ) = @_;

    my (
        $Core_Default_DBCONNECT,       $Core_Default_BASEDIR,
        $Core_Default_CORE_INSTALLDIR, $Core_Default_PORTBASE,
        $Core_Default_TMPDIR
      )
      = &wm_xml_handler::return_core_defaults;

    my ($executable_path) = join "/", $Core_Default_BASEDIR,
      $Core_Default_CORE_INSTALLDIR, "pm/adm";

    if ( -e "$executable_path/register" ) {

        my (@call_return) =
          system(
            "$executable_path/register $process_literal  $file_location");
    }
}
 
sub clean_up_files ( ) {
###################################################################
# clean_up_files : Remove the extraneous temporary files that we  #
# have created thus far.                                          #
###################################################################
    my @clean_files = (
        "/tmp/untar.log", "/tmp/process.txt",
        "/tmp/core.spec", "/tmp/modules.spec"
    );

    for ( my $x = 0 ; $x <= $#clean_files ; $x++ ) {
        my $file_to_remove = $clean_files[$x];
        if ( -e "$file_to_remove" ) {
            unlink "$file_to_remove";
        }
    }
}

1;
