#!/bin/ksh
#

########################################################
# set paths and files

MODULE="ProspectBase"
RELEASE="6.0.9.1.0"    
RP=$(echo $RELEASE|sed 's/\.//g')

SourceDir="${FLEXPM_HOME}/patches/${RELEASE}"
WorkDir=$SourceDir/workdir
TarDir=${SourceDir}/tarball/${MODULE}_${RP}_UI.tgz

### stamping ts
s_yyyymmdd=`(date "+%Y%m%d")`
s_hhmmss=`(date "+%H%M%S")`
mkdir -p $SourceDir/Backup
echo "$s_yyyymmdd.$s_hhmmss" > $SourceDir/Backup/.ts.txt

### get ts
TS=`cat $SourceDir/Backup/.ts.txt`
BACKUP_DIR=$SourceDir/Backup/$TS

########################################################

# Check env settings
 echo "Checking environment....."

  if ! [[ -d ${FLEXPM_HOME} ]]; then
     echo "Env variable FLEXPM_HOME must be set to appropriate dir before installation."
     exit;
  fi

  if ! [[ -d ${NE_BASE} ]]; then
     echo "Env variable NE_BASE must be set to appropriate dir before installation."
     exit;
  fi

  if [ -z ${DB_CONNECT} ]; then
     echo "Env variable DB_CONNECT must be set to approriate database connection values."
     exit 1;
  fi

    #######################################################
    # Installing module...
    #######################################################

    date +"+ Installing vendor specific $MODULE $RELEASE at %T, %D ..."

    echo "+ Backing up original files and installing new files..."
    cd $NE_BASE
    mkdir -p ${BACKUP_DIR}

    ${WorkDir}/untar.pl $SourceDir $TarDir $NE_BASE $BACKUP_DIR
    ret=$?

    if [ $ret != 0 ]; then
      echo "+++ Error in untar.pl"
      exit 2
    fi

    echo "+ Updating files in ${FLEXPM_HOME}/vendor/Common/NE/1.3/scripts"

    if ! [ -d ${FLEXPM_HOME}/vendor/Common/NE/1.3/scripts ] ; then
       echo "Warning: Cant locate ${FLEXPM_HOME}/vendor/Common/NE/1.3/scripts"
       exit 1
    fi

    cd ${FLEXPM_HOME}/vendor/Common/NE/1.3/scripts

    if [[ ! -f file_decode_lucentumts.pl && ! -f filedecode.pl ]]; then
       echo "Warning: Cant locate ${FLEXPM_HOME}/vendor/Common/NE/1.3/scripts/filedecode.pl to rename"
    elif [[ ! -f file_decode_lucentumts.pl && -f filedecode.pl ]]; then
       mv filedecode.pl file_decode_lucentumts.pl
       echo "+ renamed vendor specific filedecode.pl to file_decode_lucentumts.pl"
    fi

    echo "+ make all"
    make all

    ###### module installation customerize here...

    date +"+ $MODULE $RELEASE upgrade is completed at %T, %D"
    echo ""

    #######################################

    echo ' Checking Installed components...'

    # Check if installed core is greater than rev 8.0.5.4 or not
    var=0

    for version in `show_installed | grep 'CORE Prospect' | sed 's/.*\([0-9]\.[0-9]\.[0-9]\.[0-9]\).*/\1/'`
    do
        if [[ `echo $version | sed 's/\.//g` -gt $var ]]
        then
            var=`echo $version | sed 's/\.//g`
        fi
    done

    if [[ $var -lt 8054 ]]
    then
        echo " Installing Core patch 8.0.5.4..."

        cd ${SourceDir}/core_hf
        inst_core_hf.sh -a 8.0.5-PROSPECT-IF0004.tar.gz

        echo "+ Core patch 8.0.5.4 installed."
        echo ''
    else
        echo 'core version is greater than 8.0.5.4, skipped.'
    fi
