#!/usr/bin/perl

####################################################
# Prepared by : Lau In Siong
# Created  on : 27 Feb 2007
# updated  on : 22 Mar 2007
# added BACKUP dir
#
####################################################

use strict;

my $TRUE = 0 ;
my $FALSE  = 1;
my $no_of_petty_err = 0;

my $gtar_dir;
my $tarhome = $ENV {"HOME"} . "/bin";
my $backup_dir;

my $source_gtar;
my $target_dir;

& on_init;
& untar;

#############################################################
sub untar
#############################################################
{
   chop (my $curdir = `pwd`);
   my $ref_mod;
   my $backup_flag = $TRUE;

   if ((! -d "$backup_dir") || (! -e "$backup_dir"))
   {
       #die ("Cant locate dir $backup_dir!\n");
       $backup_flag = $FALSE;
   }

   chdir ($target_dir)  || die ("Cant locate dir $target_dir!\n");

   if ($backup_flag == $TRUE)
   { 
      printf ("untarring <$source_gtar> to <$target_dir>, backup at <$backup_dir>...\n");
   }
   else
   {
      printf ("untarring <$source_gtar> to <$target_dir>...\n");
   }

   my @tmp = `$gtar_dir/gtar -tzf $source_gtar`;
   chop (my $basename_source_gtar= `basename $source_gtar`);
   my $archive = sprintf ("archive.%s", $basename_source_gtar);
   foreach (@tmp)
   {
      chop ($_);
      if (( -f "$_") && ( -e "$_") )
      {
         my $mode = (stat($_))[2];
         my $ret_mod = sprintf "%04o", $mode & 07777;
         $ref_mod -> {$_} = $ret_mod;

         if ($backup_flag == $TRUE)
         {
            my $dir_name = `dirname $_`;
            #printf ("DIR NAME $dir_name\n");

            if ( ! -d "$backup_dir/$dir_name" )
            {
               #printf ("creating dir...\n");
               `mkdir -p $backup_dir/$dir_name`;
            }
            `cp $_ $backup_dir/$dir_name`;
         }

         `chmod u+w $_`; 
      }
   }

   `$gtar_dir/gtar -xvzf $source_gtar`;

   foreach my $touchfile ( keys  % { $ref_mod } )
   {
      `chmod $ref_mod->{$touchfile} $touchfile`;
   }

   chdir ($backup_dir);

   ### 20080220 (Lau): gtar the Backup dir as well
   if ($backup_flag == $TRUE)
   {
      `mv archive.* .. > /dev/null 2>&1`;
      `$gtar_dir/gtar czf ../$archive .`;
      `rm -fr *`;
      `mv ../archive.* .`;
   }
   ### End 20080220
}


#############################################################
sub on_init
#############################################################
{

   & show_usage if  (! $ARGV[0] ||  ! $ARGV[1] || ! $ARGV[2] ) ;
   $gtar_dir    = $ARGV[0];
   $source_gtar = $ARGV[1];
   $target_dir = $ARGV[2];
   $backup_dir = $ARGV[3];
 
   #chop ($start_yyyymmdd = `(date "+%Y%m%d")`);
   #chop ($start_hhmmss   = `(date "+%H%M%S")`);

   #$wkdir = sprintf ("wk.%s.%s", $start_yyyymmdd, $start_hhmmss);

   ### chkif gtar exists?
   if (! -e "$gtar_dir/gtar")
   {
      chop ($gtar_dir = `which gtar`);
      die ("Cant locate gtar dir!\n") if (! -e $gtar_dir);
      chop ($gtar_dir = `dirname $gtar_dir`);
      printf ("Use gtar from $gtar_dir instead.\n");
   }

   $TRUE = 0;
   $FALSE = 1;
}



##############################################################
sub load2buffer
##############################################################
{
   my ($filename, $opt) = @_;
   my @in;
   my @out = <NULL>;

   open (FILE, "$filename") || die ("load2buffer: Can't open file $filename for reading!\n");

   @in = <FILE>;
   foreach my $file (@in)
   {
      if ($opt == 1)
      {
        $file =~ s/^[ \t]+//;
        $file =~ s/[ \t]*\n$//;
      }
      #if (  ($file !~ /^#/) && (length($file)>0)  )
      #{
         push (@out, $file);
      #}
   }
   return @out;
}

##############################################################
sub chop_whitespace
##############################################################
{
   my ($strmsg) = @_;
   my $outmsg;

#   $strmsg =~ s/^[ \t]+//; # Remove any whitespace at the beginning of the line
#   $strmsg =~ s/[ \t]+\n$//;  # remove any whitespace at the end of line
#   #$strmsg =~ s/[ \t]+/ /g;

   $strmsg =~ s/^\s+//;
   $strmsg =~ s/\s+$//;
   $strmsg =~ s/\s+/ /g;


   return $strmsg;

}

#####################################################
sub show_usage
#####################################################
{
   die ("Usage: $0 gtardir gtarball destdir [backupdir]
         gtardir  : where the gtar executable located
         gtarball : where gtar file name (full path)
         destdir  : where destination to be untarred.
         backupdir: Optional, where the orginial files to be backed up at.
         \n");
}


##############################################################
sub petty_err
##############################################################
{
   my $strmsg = shift;

   #printf ("PETTY ERROR: \"%s\n", $strmsg);
   &write_log ("PETTY ERROR: $strmsg\n");
   $no_of_petty_err++;
}

###########################################################
sub write_log
###########################################################
{
   my ($strmsg) = @_;

   print "$strmsg";
   print LOGFILE $strmsg;
}

############################################################
sub fatal_err
###########################################################
{
   my ($strmsg) = @_;

   &write_log ("FATAL ERROR: $strmsg\n");
   die ("Exit with died");
   #die;
}
