#!/bin/ksh
###########################################################################
#
# Licensed Materials - Property of IBM
# 5724-T05
#
# (C) Copyright IBM Corp. 2003,2009. All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
###########################################################################

MODULE="LuUMTS_CS"
RELEASE="6.0.9.3.0"    

SourceDir="${FLEXPM_HOME}/patches/${RELEASE}/${MODULE}"
utilDir="${FLEXPM_HOME}/patches/${RELEASE}/utils"
RP=$(echo $RELEASE|sed 's/\.//g')
TarDir=${SourceDir}/tarball/${MODULE}_${RP}_UI.tgz

mkdir -p ${SourceDir}/Backup
TS=`date +"%Y%m%d.%H%M%S"`
BACKUP_DIR=${SourceDir}/Backup/${TS}


########################################################
# Check environment settings
echo "Checking environment....."

if ! [[ -d ${FLEXPM_HOME} ]]; then
  echo "Env variable FLEXPM_HOME must be set to appropriate dir before installation."
  exit 3;
fi

if ! [[ -d ${FLEXPM_BASE} ]]; then
  echo "Env variable FLEXPM_BASE must be set to appropriate dir before installation."
  exit 3;
fi

if [ -z ${DB_CONNECT} ]; then
  echo "Env variable DB_CONNECT must be set to appropriate database connection values."
  exit 3;
fi

if ! [[ -d ${LuUMTS_CS_HOME} ]]; then
  echo "Env variable LuUMTS_CS_HOME must be set to appropriate dir before installation."
  exit 3;
fi

#######################################################
# Installing updated files ...
#######################################################
date +"+ Installing Patch ${MODULE} ${RELEASE} at %T, %D ..."

echo "+ Backing up original files and installing new files..."

mkdir -p ${BACKUP_DIR}
${utilDir}/untar.pl ${SourceDir} ${TarDir} ${FLEXPM_BASE} ${BACKUP_DIR}
ret=$?

if [ $ret != 0 ]; then
  echo "+++ Error in untar.pl"
  exit 2
fi


#######################################################
# Perform custom actions ...
#######################################################

# Perform the RP9.2 post install action. This won't cause any issues if it has already
# been run.
echo "exec CreateSumTab.CreateSummaryTables('D', 'LU_TF_U_CS_LINKSET_TAB')" | sqlplus -s ${DB_CONNECT} 
echo "@${FLEXPM_HOME}/patches/${RELEASE}/utils/ext_add_shadow_tables.sql" | sqlplus -s ${DB_CONNECT}
echo "exec ext_add_shadow_tables('LU_TF_U_CS_LINKSET_TAB')" | sqlplus -s ${DB_CONNECT}

# 6.0.9.2.1 - WM_CS_SM_FIND_INSTANCE issue.
echo "+ Updating the find instance package ..."
echo "@${LuUMTS_CS_HOME}/vendor/Lucent/UMTS_CS/Network/wm_cs_sm_find_instance_pkg.sql" | sqlplus -s ${DB_CONNECT}
echo "@${LuUMTS_CS_HOME}/vendor/Lucent/UMTS_CS/Network/wm_cs_sm_find_instance_pkgb.sql" | sqlplus -s ${DB_CONNECT}

# Perform a dict_update to load the latest data dictionary into the database.
echo "+ Updating the data dictionary ..."
cd ${LuUMTS_CS_HOME}/peik
make dict_update
if [ $? != 0 ]; then
  echo "+++ Error encountered during the data dictionary update."
  echo "+++ Please consult with IBM support for advice"
  exit 3
fi

# Plug in the updated loader configuration
echo "+ Updating the loader configuration ..."
cd ${LuUMTS_CS_HOME}/vendor/Lucent/UMTS_CS/1.3/LuUMTS_CS_data
make unplug
make plug

# Plug in the updated makeArg/makeSentry plugin 
echo "+ Updating the makeArg/makeSentry CS plugin ..."
cd ${LuUMTS_CS_HOME}/vendor/Lucent/UMTS_CS/1.3/scripts
make setMod
make unplug
make setMod
make plug
chmod +x ${FLEXPM_HOME}/vendor/Common/NE/1.3/scripts/makeArg
chmod +x ${FLEXPM_HOME}/vendor/Common/NE/1.3/scripts/makeSentry

# Create the ftpIN directories for LNG and LMRS
echo "+ Making new ftpIN directories ..."
./mkdirs.sh

# Link the new directories into place
echo "+ Linking new ftpIN directories ..."
ln -f ${LuUMTS_CS_HOME}/vendor/Lucent/UMTS_CS/ftpIN/LNG ${FLEXPM_HOME}/vendor/Common/NE/ftpIN/LNG
ln -f ${LuUMTS_CS_HOME}/vendor/Lucent/UMTS_CS/ftpIN/LMRS ${FLEXPM_HOME}/vendor/Common/NE/ftpIN/LMRS

# Build the loader configuration for the LMRS loader
echo "+ Building the LMRS loader ..."
cd ${LuUMTS_CS_HOME}/vendor/Lucent/UMTS_CS/1.3/LuUMTS_CS_loader/LMRS_pm
make all

# Build the loader configuration for the LNG loader
echo "+ Building the LNG loader ..."
cd ${LuUMTS_CS_HOME}/vendor/Lucent/UMTS_CS/1.3/LuUMTS_CS_loader/LNG_pm
make all

# If an element_list exists in the system, then rebuild the sentry and argument file so that 
# the LMRS and LNG data will be picked up by the sentry.
if [ -s ${FLEXPM_HOME}/vendor/Common/NE/1.3/element_list ]; then
  echo "+ Updating the sentry patterns ..."
  cd ${FLEXPM_HOME}/vendor/Common/NE/1.3/data
  cp Sentry.txt Sentry.txt.9.3bak
  cp Argument.txt Argument.txt.9.3bak
  chmod +w Sentry.txt Argument.txt
  makeArg ../element_list > Argument.txt
  makeSentry ../element_list
fi

# Rebuild the available data cleaning routines with the extra LNG and LMRS patterns.
echo "+ Resetting plugin for missingData ..."
cd ${LuUMTS_CS_HOME}/vendor/Lucent/UMTS_CS/missingData
make unplug
make plug

# Add the extra package required for updating the names/labels and for creating scenarios
# for the LMRS data
echo "+ Creating GTT for LMRS data ..."
cd ${LuUMTS_CS_HOME}/vendor/Lucent/UMTS_CS/Network
if [ -f create_tmp_cs_update_name_label.o ]; then
   echo "touched create_tmp_cs_update_name_label.o"
   touch create_tmp_cs_update_name_label.o
fi
make all

echo "+ Updating FI metadata ..."
echo "@ wm_Activate_Parent.sql" | sqlplus -s ${DB_CONNECT}
echo "@ wm_validate_fullkey.sql" | sqlplus -s ${DB_CONNECT}

# Rebuild the loaders for the RNCpm data. This was modified to work with wml_load, but the
# code changes never released in an official release. They are now included and the loaders
# need to be rebuild to use the new configuration.
echo "+ Rebuilding the CallServer0302_pm loaders ..."
cd ${LuUMTS_CS_HOME}/vendor/Lucent/UMTS_CS/1.3/LuUMTS_CS_loader/CallServer0302_pm/data
chmod +w LoadAction.txt
cd ${LuUMTS_CS_HOME}/vendor/Lucent/UMTS_CS/1.3/LuUMTS_CS_loader/CallServer0302_pm/config
rm -f LoadAction.txt
chmod +x rewrite_dbtable.pl build_load_action.pl
make clean
make all

# Rebuild the scenario queries for A and C types.
echo "+ Adding new entities into existing system Active/Complete scenarios ..."
cd ${utilDir}
echo "@ regen_sce_query.sql" | sqlplus -s $DB_CONNECT

#######################################################
# Patching is complete ...
#######################################################
date +"+ ${MODULE} ${RELEASE} upgrade is completed at %T, %D"
echo 

