#!/bin/ksh
###########################################################################
#
# Licensed Materials - Property of IBM
# 5724-T05
#
# (C) Copyright IBM Corp. 2003,2009. All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
###########################################################################

MODULE="LuUTRAN"
RELEASE="6.0.9.3.0"    

SourceDir="${FLEXPM_HOME}/patches/${RELEASE}/${MODULE}"
utilDir="${FLEXPM_HOME}/patches/${RELEASE}/utils"
RP=$(echo $RELEASE|sed 's/\.//g')
TarDir=${SourceDir}/tarball/${MODULE}_${RP}_UI.tgz

mkdir -p ${SourceDir}/Backup
TS=`date +"%Y%m%d.%H%M%S"`
BACKUP_DIR=${SourceDir}/Backup/${TS}


########################################################
# Check environment settings
echo "Checking environment....."

if ! [[ -d ${FLEXPM_HOME} ]]; then
  echo "Env variable FLEXPM_HOME must be set to appropriate dir before installation."
  exit 3;
fi

if ! [[ -d ${FLEXPM_BASE} ]]; then
  echo "Env variable FLEXPM_BASE must be set to appropriate dir before installation."
  exit 3;
fi

if [ -z ${DB_CONNECT} ]; then
  echo "Env variable DB_CONNECT must be set to appropriate database connection values."
  exit 3;
fi

if ! [[ -d ${LuUTRAN_HOME} ]]; then
  echo "Env variable LuUTRAN_HOME must be set to appropriate dir before installation."
  exit 3;
fi

#######################################################
# Installing updated files ...
#######################################################
date +"+ Installing Patch $MODULE $RELEASE at %T, %D ..."

echo "+ Backing up original files and installing new files..."

mkdir -p ${BACKUP_DIR}
${utilDir}/untar.pl $SourceDir $TarDir $FLEXPM_BASE $BACKUP_DIR
ret=$?

if [ $ret != 0 ]; then
  echo "+++ Error in untar.pl"
  exit 2
fi


#######################################################
# Perform custom actions ...
#######################################################

# Rebuild the data dictionary
cd ${LuUTRAN_HOME}/peik
make dict_update
ret=$?
if [ $ret != 0 ]; then
  echo "+++ Error while performing dict_update!"
  exit 2
fi

# check if it is a broken UNTRANcm loader?
sqlplus $DB_CONNECT @${utilDir}/validate_wmload.sql LU_TF_U_UTRANCFG
ret=$?
if [ $ret -lt 1 ]; then
   cd ${LuUTRAN_HOME}/vendor/Lucent/UTRAN/1.3/LuUTRAN_loader/UTRANcm/config
   make clean
   make all
   ret2=$?
   if [ $ret2 != 0 ]; then
     echo "+++ Error while building UTRANcm loader!"
     exit 2
   fi
fi


# check if it is a broken MATRIXpm loader?
sqlplus $DB_CONNECT @${utilDir}/validate_wmload.sql LU_TF_U_MATRIX
ret=$?
if [ $ret -lt 1 ]; then
   cd ${LuUTRAN_HOME}/vendor/Lucent/UTRAN/1.3/LuUTRAN_loader/RNC_MATRIXpm/config
   make clean
   make all
   ret2=$?
   if [ $ret2 != 0 ]; then
     echo "+++ Error while building RNC_MATRIXpm loader!"
     exit 2
   fi
fi

# Update the jloader for RNC CN to load the new VS.RESERVED counters.
cd ${LuUTRAN_HOME}/vendor/Lucent/UTRAN/1.3/LuUTRAN_loader/RNC_CN/config
make build_loaders
ret=$?
if [ $ret != 0 ]; then
  echo "+++ Error while performing jloader configuration update!"
  exit 2
fi 

# Update the RNC05 loader for additional counter to load - VS.MM.PagAttDiscard.sum
cd ${LuUTRAN_HOME}/vendor/Lucent/UTRAN/1.3/LuUTRAN_loader/RNCpm/config
rm DatabaseFile.txt DatabaseTable.txt LoaderControl.txt
make build_loaders
ret=$?
if [ $ret != 0 ]; then
  echo "+++ Error while performing jloader configuration update!"
  exit 2
fi 

# Install new procedure for valnt00088737
cd ${LuUTRAN_HOME}/vendor/Lucent/UTRAN/Network
echo "@ wm_luc_utran_create_scenario.sql" | sqlplus -s $DB_CONNECT

#######################################################
# Patching is complete ...
#######################################################
date +"+ $MODULE $RELEASE upgrade is completed at %T, %D"
echo 

