#!/bin/ksh
###########################################################################
#
# Licensed Materials - Property of IBM
# 5724-T05
#
# (C) Copyright IBM Corp. 2003,2009. All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
###########################################################################

MODULE="VendorCommon"
RELEASE="6.0.9.3.0"    

SourceDir="${FLEXPM_HOME}/patches/${RELEASE}/${MODULE}"
utilDir="${FLEXPM_HOME}/patches/${RELEASE}/utils"
RP=$(echo $RELEASE|sed 's/\.//g')
TarDir=${SourceDir}/tarball/${MODULE}_${RP}_UI.tgz

mkdir -p ${SourceDir}/Backup
TS=`date +"%Y%m%d.%H%M%S"`
BACKUP_DIR=${SourceDir}/Backup/${TS}


########################################################
# Check environment settings
echo "Checking environment....."

if ! [[ -d ${FLEXPM_HOME} ]]; then
  echo "Env variable FLEXPM_HOME must be set to appropriate dir before installation."
  exit 3;
fi

if ! [[ -d ${FLEXPM_BASE} ]]; then
  echo "Env variable FLEXPM_BASE must be set to appropriate dir before installation."
  exit 3;
fi

if [ -z ${DB_CONNECT} ]; then
  echo "Env variable DB_CONNECT must be set to appropriate database connection values."
  exit 3;
fi

#######################################################
# Installing updated files ...
#######################################################
date +"+ Installing Patch ${MODULE} ${RELEASE} at %T, %D ..."

echo "+ Backing up original files and installing new files..."

mkdir -p ${BACKUP_DIR}
${utilDir}/untar.pl ${SourceDir} ${TarDir} ${FLEXPM_BASE} ${BACKUP_DIR}
ret=$?

if [ $ret != 0 ]; then
  echo "+++ Error in untar.pl"
  exit 2
fi


#######################################################
# Perform custom actions ...
#######################################################

# Make sure the file_decode_lucentumts.pl script is executable
chmod 555 ${FLEXPM_HOME}/vendor/Common/NE/1.3/scripts/file_decode_lucentumts.pl

# Install the updated mappings for the techpart map partitioning.
echo "@ ${FLEXPM_HOME}/vendor/Common/NE/1.3/scripts/techpart/insert_techpart_map.sql" | sqlplus -s $DB_CONNECT

#######################################################
# Patching is complete ...
#######################################################
date +"+ ${MODULE} ${RELEASE} upgrade is completed at %T, %D"
echo 

